<?php
/**
 * Created by PhpStorm.
 * User: fffff
 * Date: 2018/11/20
 * Time: 14:48
 */

namespace App\Services\Common;

use App\Constants\CommonRespCode;
use App\Exceptions\AuthenticateException;
use App\Exceptions\BusinessException;
use App\Exceptions\CurlException;
use App\Libraries\Helper\Request;
use Illuminate\Support\Facades\Log;

class UserInfo
{

    private static $instance = null;

    private $userId;

    private $token;

    private $platform;//1福利小程序 2福利后台

    private $userInfo;


    public static function getInstance()
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function setUserInfo(array $userInfo)
    {
        $this->userInfo = $userInfo;
    }

    public function getUserInfo()
    {
        return $this->userInfo;
    }

    public function setPlatform($platform)
    {
        $this->platform = $platform;
        return $this;
    }

    public function setUserId($userId)
    {
        $this->userId = $userId;
        return $this;
    }

    public function getUserId()
    {
        return $this->userId;
    }

    public function setToken($token)
    {
        $this->token = $token;
        return $this;
    }

    /**
     * 从财务系统获取用户积分情况
     * @return array
     * @throws CurlException
     */
    public function getUserFromFinance()
    {
        $url = config('api.FXQF_FINANCE') . config('api.FXQF_FINANCE_GET_USER_INFO');
        $result = Request::get($url, ['user_id' => $this->userId]);
        Log::channel('userinfo')->info(
            vsprintf('【用户积分信息获取】请求url：[%s],响应结果：[%s]', [
                $url, var_export($result, true)
            ])
        );
        $result = json_decode($result, true);
        if (0 !== $result['code']) {
            return ['balance' => 0, 'total_income_amount' => 0];
        }
        $result = $result['data'];
        $result['balance'] = $result['balance'] ?? 0;
        $result['total_income_amount'] = $result['total_income_amount'] ?? 0;
        return [
            'balance' => round($result['balance'], 2),
            'total_income_amount' => round($result['total_income_amount'], 2)
        ];
    }

    /**
     * 获取用户信息
     * @return array|mixed|string
     * @throws AuthenticateException
     * @throws BusinessException
     */
    public function getUserFromUserCenterWithToken()
    {
        $url = config('api.USER_CENTER') . config('api.USER_CENTER_GET_USER_INFO');
        $data = [
            'fx-token' => $this->token,
            'get_type' => UserCenter::USER_CENTER_GET_WITH_TOKEN_TYPE,
            'method' => UserCenter::USER_CENTER_GET_ONE_USER_METHOD,
            'platform' => 1, //TODO 临时
        ];
        try {
            $result = Request::get($url, $data);
        } catch (CurlException $e) {
            throw new BusinessException(CommonRespCode::CURL_ERROR);
        }
        Log::channel('userinfo')->info(
            vsprintf('【用户基础信息获取】请求url：[%s],响应结果：[%s]', [
                $url, var_export($result, true)
            ])
        );

        $result = json_decode($result, true);

        if (0 !== $result['code']) {
            throw new AuthenticateException();
        }
        $result = [
            'nick_name' => $result['data']['nick_name'] ?? '',
            'avatar' => $result['data']['avatar'] ?? '',
            'section_name' => $result['data']['section_name'] ?? '',
            'cover' => $result['data']['cover'] ?? '',
            'company_name' => $result['data']['company_name'] ?? '',
            'company_id' => $result['data']['company_id'] ?? '',
            'phone' => $result['data']['phone'] ?? '',
            'uid' => $result['data']['uid'] ?? '',
            'name' => $result['data']['staff_name'] ?? '',
            'staff_id' => $result['data']['staff_id'] ?? '',
        ];
        return $result;
    }

    /**
     * 获取用户信息
     * @return array|mixed|string
     */
    public function getUserFromUserCenterWithUserId()
    {
        $url = config('api.USER_CENTER') . config('api.USER_CENTER_GET_USER_INFO');
        $data = [
            'uid' => $this->userId,
            'get_type' => UserCenter::USER_CENTER_GET_WITH_UID_TYPE,
            'method' => UserCenter::USER_CENTER_GET_ONE_USER_METHOD,
            'platform' => 1,//TODO  临时
        ];
        try {
            $result = Request::get($url, $data);
            Log::channel('userinfo')->info(
                vsprintf('【用户基础信息获取】请求url：[%s],响应结果：[%s]', [
                    $url, var_export($result, true)
                ])
            );
            $result = json_decode($result, true);

        } catch (CurlException $e) {
            return [
                'nick_name' => '',
                'avatar' => '',
                'section_name' => '',
                'cover' => '',
                'company_name' => '',
                'company_id' => '',
                'phone' => '',
                'uid' => '',
                'name' => '',
                'staff_id' => ''
            ];
        }
        $result = [
            'nick_name' => $result['data']['nick_name'] ?? '',
            'avatar' => $result['data']['avatar'] ?? '',
            'section_name' => $result['data']['section_name'] ?? '',
            'cover' => $result['data']['cover'] ?? '',
            'company_name' => $result['data']['company_name'] ?? '',
            'company_id' => $result['data']['company_id'] ?? '',
            'phone' => $result['data']['phone'] ?? '',
            'uid' => $result['data']['uid'] ?? '',
            'name' => $result['data']['staff_name'] ?? '',
            'staff_id' => $result['data']['staff_id'] ?? '',
        ];
        return $result;
    }

    /**
     * 获取多个用户的信息
     * @param array $user_ids
     * @return array
     */
    public function getUsersFromUserCenter(array $user_ids)
    {
        if (empty($user_ids)) {
            return [];
        }
        $url = config('api.USER_CENTER') . config('api.USER_CENTER_GET_USERS_INFO');
        $data = [
            'uid' => implode(',',$user_ids)
        ];
        try
        {
            $result = Request::get($url, $data);
            Log::channel('userinfo')->info(
                vsprintf('【用户基础信息获取】请求url：[%s], 响应结果：[%s]', [
                    $url, var_export($result, true)
                ])
            );
            $result = json_decode($result, true);
            if (0 !== $result['code']) {
                return [];
            }
            $result = $result['data']['list'];

            $users = [];
            if (!empty($result) && is_array($result)) {
                foreach ($result as $key => $val) {
                    $users[$val['uid']] = $val;
                }
            }
            return $users;
        }catch (CurlException $e) {
            return [];
        }
    }

    /**
     * 根据条件筛选用户
     * @param $company_id
     * @param $condition
     * @param $keyword
     * @return array|mixed|string
     */
    public function getUsersByKeywordFromUserCenter($company_id, $condition, $keyword)
    {
        $url = config('api.USER_CENTER') . config('api.USER_CENTER_GET_USER_INFO_BY_KEYWORD');
        $data = [
            'company_id' => $company_id,
            'condition' => $condition,
            'keyword' => $keyword,
        ];
        try {
            $result = Request::get($url, $data);
            Log::channel('userinfo')->info(
                vsprintf('【用户基础信息获取】请求url：[%s], data:%s 响应结果：[%s]', [
                    $url, json_encode($data), var_export($result, true)
                ])
            );
            $result = json_decode($result, true);
            if (0 !== $result['code']) {
                return [];
            }
            $result = $result['data']['list'];
            return $result;
        } catch (CurlException $e) {
            return [];
        }
    }
    /**
     * 获取所有公司
     * @author yujie
     * @date 2018-11-22
     * @param $condition
     * @param $keyword
     * @return mixed|string
     * @throws BusinessException
     */
    public function getCompanies()
    {
        $url = config('api.USER_CENTER') . config('api.USER_CENTER_GET_COMPANIES');
        try
        {
            $result = Request::get($url);
            Log::channel('userinfo')->info(
                vsprintf('【用户基础信息获取】请求url：[%s], 响应结果：[%s]', [
                    $url, var_export($result, true)
                ])
            );
            $result = json_decode($result, true);
            if (0 !== $result['code']) {
                return [];
            }
            $result = $result['data'];
            return $result;
        }catch (CurlException $e) {
            return [];
        }
    }
}
