<?php
/**
 * Created by PhpStorm.
 * User: fffff
 * Date: 2018/12/10
 * Time: 15:43
 */

namespace App\Services\Common;

use App\Exceptions\BusinessException;
use App\Exceptions\SystemException;
use EasyWeChat\Factory;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Log;

class WeChat
{

    protected static $instance;

    protected $cache;

    protected $token;

    protected $easyWeChat;

    /**
     * 获得句柄
     * @return WeChat
     */
    public static function getInstance()
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function getEasyWeChatInstance()
    {
        return $this->easyWeChat;
    }

    public function __construct()
    {
        $this->cache = Cache::store('redis');
        $this->easyWeChat = Factory::officialAccount(config('weChat'));
    }


    /**
     * @return mixed
     * @throws SystemException
     * @throws \Psr\SimpleCache\InvalidArgumentException
     */
    public function getToken()
    {
        $keys = $this->cache->keys(COMPANY_SOURCE . '-TOKEN-*');
        if (empty($keys)) {
            $this->setToken();
        } else {
            $key = array_pop($keys);
            $expire = $this->cache->ttl($key);

            if ($expire <= 7200) {
                $this->setToken();
            } else {
                $this->token = $this->cache->get($key);
                $this->easyWeChat->access_token->setToken($this->token, $expire);
            }
        }
        return $this->token;
    }

    /**
     * 设置token
     * @throws SystemException
     * @throws \Psr\SimpleCache\InvalidArgumentException
     */
    protected function setToken()
    {
        $keyName = COMPANY_SOURCE . '-TOKEN-' . time();
        try {
            $token = $this->easyWeChat->access_token->getToken();
            $this->cache->set($keyName, $token, 7200);
        } catch (\Exception $e) {
            Log::channel('syslog')->info('缓存连接失败');
            throw new SystemException('系统繁忙');
        }
        $this->token;
    }

}
